use rand::Rng;
use rusqlite::Row;

use super::{DbError, DbPool, User};

pub struct Session {
    pub id: i64,
    pub user_id: i32,
}

impl Session {
    pub fn from_row(row: &Row<'_>) -> Result<Self, rusqlite::Error> {
        Ok(Self {
            id: row.get("id")?,
            user_id: row.get("user_id")?,
        })
    }

    pub fn user_from_id(pool: &DbPool, id: i64) -> Result<Option<super::User>, DbError> {
        let conn = pool.get()?;

        let mut stmt = conn.prepare("select users.* from sessions inner join users on sessions.user_id = users.id where sessions.id = $1")?;
        match stmt.query_row((id,), User::from_row) {
            Ok(user) => Ok(Some(user)),
            Err(rusqlite::Error::QueryReturnedNoRows) => Ok(None),
            Err(err) => Err(DbError::Db(err)),
        }
    }

    pub fn new_for_user(pool: &DbPool, user_id: i64) -> Result<Self, DbError> {
        let id: i64 = rand::thread_rng().gen();

        let conn = pool.get()?;
        let mut stmt =
            conn.prepare("insert into sessions (id, user_id) values ($1, $2) returning *")?;

        Ok(stmt.query_row((&id, &user_id), Self::from_row)?)
    }
}
